/* pam_strerror.c */

/*
 * $Id: pam_strerror.c,v 1.2 2000/12/04 19:02:34 baggins Exp $
 */

#include "pam_private.h"

const char *pam_strerror(pam_handle_t *pamh, int errnum)
{
#ifdef UGLY_HACK_FOR_PRIOR_BEHAVIOR_SUPPORT  /* will be removed from v 1.0 */

    int possible_error;

    possible_error = (int) pamh;
    if (!(possible_error >= 0 && possible_error <= PAM_BAD_ITEM)) {
	possible_error = errnum;
    }

/* mask standard behavior to use possible_error variable. */
#define errnum possible_error

#endif /* UGLY_HACK_FOR_PRIOR_BEHAVIOR_SUPPORT */

    switch (errnum) {
    case PAM_SUCCESS:
	return "Success";
    case PAM_ABORT:
	return "Critical error - immediate abort";
    case PAM_OPEN_ERR:
	return "dlopen() failure";
    case PAM_SYMBOL_ERR:
	return "Symbol not found";
    case PAM_SERVICE_ERR:
	return "Error in service module";
    case PAM_SYSTEM_ERR:
	return "System error";
    case PAM_BUF_ERR:
	return "Memory buffer error";
    case PAM_PERM_DENIED:
	return "Permission denied";
    case PAM_AUTH_ERR:
	return "Authentication failure";
    case PAM_CRED_INSUFFICIENT:
	return "Insufficient credentials to access authentication data";
    case PAM_AUTHINFO_UNAVAIL:
	return "Authentication service cannot retrieve authentication info.";
    case PAM_USER_UNKNOWN:
	return "User not known to the underlying authentication module";
    case PAM_MAXTRIES:
	return "Have exhasted maximum number of retries for service.";
    case PAM_NEW_AUTHTOK_REQD:
	return "Authentication token is no longer valid; new one required.";
    case PAM_ACCT_EXPIRED:
	return "User account has expired";
    case PAM_SESSION_ERR:
	return "Cannot make/remove an entry for the specified session";
    case PAM_CRED_UNAVAIL:
	return "Authentication service cannot retrieve user credentials";
    case PAM_CRED_EXPIRED:
	return "User credentials expired";
    case PAM_CRED_ERR:
	return "Failure setting user credentials";
    case PAM_NO_MODULE_DATA:
	return "No module specific data is present";
    case PAM_BAD_ITEM:
	return "Bad item passed to pam_*_item()";
    case PAM_CONV_ERR:
	return "Conversation error";
    case PAM_AUTHTOK_ERR:
	return "Authentication token manipulation error";
    case PAM_AUTHTOK_RECOVER_ERR:
	return "Authentication information cannot be recovered";
    case PAM_AUTHTOK_LOCK_BUSY:
	return "Authentication token lock busy";
    case PAM_AUTHTOK_DISABLE_AGING:
	return "Authentication token aging disabled";
    case PAM_TRY_AGAIN:
	return "Failed preliminary check by password service";
    case PAM_IGNORE:
	return "Please ignore underlying account module";
    case PAM_MODULE_UNKNOWN:
	return "Module is unknown";
    case PAM_AUTHTOK_EXPIRED:
	return "Authentication token expired";
    case PAM_CONV_AGAIN:
	return "Conversation is waiting for event";
    case PAM_INCOMPLETE:
	return "Application needs to call libpam again";
    case PAM_PWERR_GETCONFIG:
	return "Failed to retrieve data from FABOS config database";
    case PAM_PWERR_LOWER:
	return "Insufficient number of lower case letters";
    case PAM_PWERR_UPPER:
	return "Insufficient number of upper case letters";
    case PAM_PWERR_CHARSET:
	return "Insufficient number of characters";
    case PAM_PWERR_DIGITS:
	return "Insufficient number of digits in password";
    case PAM_PWERR_PUNCT:
	return "Insufficient number of punctuation characters";
    case PAM_PWERR_MINLEN:
	return "Invalid length of password";
    case PAM_PWERR_HISTORY:
	return "Password matches one of the previous passwords";
    case PAM_PWERR_MINAGE:
	return "Password cannot be changed before minimum aging period has elapsed";
    case PAM_ACCT_LOCKED:
	return "Account locked due to failed login attempts";
    case PAM_PWERR_INVALID_CHAR:
	return "Password contains invalid characters";
    case PAM_INVALID_AD_ERR:
	return "Cannot assign valid AD at the login session";
    case PAM_INVALID_VF_ERR:
	return "Cannot assign valid VF at the login session";
    case PAM_PWERR_REPEAT:
	return "Password contains more repeated characters than allowed";
    case PAM_PWERR_SEQUENCE:
	return "Password contains more character sequences than allowed";
    case PAM_PWERR_REVERSE:
	return "Password cannot be reverse of username";
    case PAM_PWERR_SAME_AS_USERNAME:
	return "Password cannot be same as username";
    case PAM_PWERR_CHKUNAME:
	return "User name cannot be part of password";
    case PAM_ACCT_DISABLED_ERR:
        return "";
    case PAM_ACCESS_DENIED:
        return "";
    case PAM_REUSE_PASSWD_DENIED:
        return "";
    }

    return "Unknown Linux-PAM error (need to upgrde libpam?)";
}
