/*
 * PAM ADLDAP module
 *
 * Description: Header file for PAM LDAP module. 
 *
 */
#ifndef PAM_ADLDAP_H
#define PAM_ADLDAP_H

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <syslog.h>
#include <stdarg.h>
#ifndef __USE_BSD
#define __USE_BSD
#include <sys/time.h>
#undef __USE_BSD
#endif /* __USE_BSD */
#include <utmp.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>


/*************************************************************************
 * Additional ADLDAP definitions
 *************************************************************************/

typedef unsigned char u_char;
typedef unsigned short u_short;

typedef struct adldap_server_t {
	struct adldap_server_t *next;
	struct in_addr ip;
	u_short port;
	char *hostname;
	int timeout;
	char *domain;
	int  bindingMethod;
	int  serverClass;
        /* This attr is used for finding DN for given attribute value */
        char *uidSearchAttr;
        /* If binding is based on configured credentail,
           clientDN and password are used for bind. */
        char *clientDN;
        char *password;

} adldap_server_t;

typedef struct adldap_conf_t {
	adldap_server_t *server;
	int retries;
	char *client_id;
	int sockfd;
	int debug;
} adldap_conf_t;

#define	LDAP_ROLE_CONF	"/etc/fabos/ldap.map"

/*************************************************************************
 * Platform specific defines
 *************************************************************************/

#ifdef sun
#define PAM_EXTERN extern
/*
 *  On older versions of Solaris, you may have to change this to:
 *  #define CONST
 */
#define CONST const
#else
#define CONST const
#endif

/*************************************************************************
 * Useful macros and defines
 *************************************************************************/

#define PAM_SM_AUTH
#define PAM_SM_PASSWORD
#define PAM_SM_SESSION
#define PORT_STR_LEN	6

#define PAM_DEBUG_ARG      1
#define PAM_SKIP_PASSWD    2
#define PAM_USE_FIRST_PASS 4
#define PAM_TRY_FIRST_PASS 8

#define	PAM_ADLDAP_UNAME		"rad_user_name"
#define	PAM_ADLDAP_LIVESERV		"adldap_live_server"
#define	PAM_ADLDAP_ADMAP		"rad_admap"
#define	PAM_ADLDAP_HOMEAD		"rad_homead"
#define	PAM_ADLDAP_VFROLEMAP	"rad_vfrolemap"
#define	PAM_ADLDAP_HOMEVF		"rad_homevf"
#define PAM_ADLDAP_FIPSMODE     "rad_fipsmode"
#define	PAM_ADLDAP_MAXVF		256
#define	PAM_ADLDAP_REAL_MAXVF		128
#define	DEFAULT_ROLE		"user"
#define	INVALID_VF				-1
#define	INVALID_ROLE			-1

#define PAM_RETRY          0x30

/* Parameters for AD List processing */
#define	ACCT_MAX_AD	256
#define BM_NBBY 8                                       /* # bits per byte */
#define BM_NBPF (sizeof (unsigned long) * BM_NBBY)      /* # bits per field */
#define BM_SetBitN(n, p)        ((p)[(n)/BM_NBPF] |= \
		((unsigned)(1 << ((n) % BM_NBPF))))
#define	BM_isBitNSet(n, p)	((p))[(n)/BM_NBPF] & \
		((unsigned)(1 << ((n) % BM_NBPF)))
#define _pam_forget(X) if (X) {memset(X, 0, strlen(X));free(X);X = NULL;}

#ifndef _pam_drop
#define _pam_drop(X) if (X) {free(X);X = NULL;}
#endif

/* Module defines */
#ifndef SMALL_BUFFER_SIZE
#define SMALL_BUFFER_SIZE      256
#endif /* SMALL_BUFFER_SIZE */
#ifndef BUFFER_SIZE
#define BUFFER_SIZE      1024
#endif /* BUFFER_SIZE */
#define MAXPWNAM 40     /* maximum user name length. Server dependent,
                         * this is the default value
                         */
#define MAXPASS 128     /* max password length. Again, depends on server
                         * compiled in. This is the default.
                         */
#ifndef CONF_FILE       /* the configuration file holding the server secret */
#define CONF_FILE       "/etc/fabos/ldap.conf"
#endif /* CONF_FILE */

#ifndef FALSE
#define FALSE 0
#undef TRUE
#define TRUE !FALSE
#endif

/* Keep this in sync with definition in fabos/src/include/security/public.h */
typedef enum {
        LDAP_SERVER_LOCAL    = 0,
        LDAP_SERVER_EXTERNAL = 1
}ldapServerType_e;

/* Keep this in sync with definition in fabos/src/include/security/public.h */
typedef enum {
        BIND_CREDENTIAL = 1,
        BIND_LOGIN_ID   = 2
}ldapBindType_e;

#endif /* PAM_ADLDAP_H */
