#ifndef PAM_RADIUS_H
#define PAM_RADIUS_H

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <syslog.h>
#include <stdarg.h>
#ifndef __USE_BSD
#define __USE_BSD
#include <sys/time.h>
#undef __USE_BSD
#endif /* __USE_BSD */
#include <utmp.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>

#include "radius.h"
#include "md5.h"

#define 	X509_V_ERR_HOSTNAME_MISMATCH 			62
#define		X509_V_ERR_CERT_UNKNOWN_TO_OCSP			65
#define		X509_V_ERR_CERT_REVOKED_TO_OCSP			66
#define		X509_V_ERR_UNABLE_TO_FIND_OCSP_URL		67
#define		X509_V_ERR_UNPARSABLE_OCSP_URL			68
#define		X509_V_ERR_SSL_USED_FOR_OCSP			69
#define		X509_V_ERR_OCSP_PROCESS_FUNC_NOT_SET		70
#define		X509_V_ERR_OCSP_RESPONDER_STATUS_FAIL		71
#define		X509_V_ERR_OCSP_NONCE_VERIFY_ERROR		72
#define		X509_V_ERR_OCSP_RESPONSE_VERIFICATION		73
#define         X509_V_ERR_OCSP_APPLICATION_VERIFICATION        74

/*************************************************************************
 * Additional RADIUS definitions
 *************************************************************************/

/* Per-attribute structure */
typedef struct attribute_t {
  unsigned char attribute;
  unsigned char length;
  unsigned char data[1];
} attribute_t;

typedef struct radius_server_t {
  struct radius_server_t *next;
  struct in_addr ip;
  u_short port;
  char *hostname;
  char *secret;
  int timeout;
  int retries;
  int accounting;
  int pap;
} radius_server_t;

typedef struct radius_conf_t {
  radius_server_t *server;
  char *client_id;
  int accounting_bug;
  int sockfd;
  int debug;
  char *remote_id;	
} radius_conf_t;


/*************************************************************************
 * Platform specific defines
 *************************************************************************/

#ifdef sun
#define PAM_EXTERN extern
/*
 *  On older versions of Solaris, you may have to change this to:
 *  #define CONST
 */
#define CONST const
#else
#define CONST const
#endif

/*************************************************************************
 * Useful macros and defines
 *************************************************************************/

#define _pam_forget(X) if (X) {memset(X, 0, strlen(X));free(X);X = NULL;}
#ifndef _pam_drop
#define _pam_drop(X) if (X) {free(X);X = NULL;}
#endif

#define PAM_DEBUG_ARG      1
#define PAM_SKIP_PASSWD    2
#define PAM_USE_FIRST_PASS 4
#define PAM_TRY_FIRST_PASS 8

#define PAM_RETRY          0x30

#define	CHAP_MD5		22	/* CHAP ID for MD5 */
#define	BRCD_VENDOR_CODE	1588	/* Brocade Private Enterprise code */
typedef	enum {
	BRCD_TYPE_AUTH_ROLE	= 1,	/* Brocade-Auth-Role */
	BRCD_TYPE_AV1,				/* Brocade-AVPairs1 */
	BRCD_TYPE_AV2,				/* Brocade-AVPairs2 */
	BRCD_TYPE_AV3,				/* Brocade-AVPairs3 */
	BRCD_TYPE_AV4,				/* Brocade-AVPairs4	*/
	BRCD_ATTR_PASSWD_EXPIREDATE,
	BRCD_ATTR_WARNPERIOD	
} brcd_vtype_e;

#define	BRCD_DEFAULT_ROLE		"user"
#define	BRCD_ATTR_ADLIST		"ADList"
#define	BRCD_ATTR_VFROLEMAP		"LFRoleList"
#define	BRCD_ATTR_HOMEAD		"HomeAD"
#define	BRCD_ATTR_HOMEVF		"HomeLF"
#define	BRCD_ATTR_CHASSISROLE	"ChassisRole"

#define	PAM_RADIUS_UNAME	"rad_user_name"
#define	PAM_RADIUS_LIVESERV	"rad_live_server"
#define	PAM_RADIUS_ADMAP	"rad_admap"
#define	PAM_RADIUS_VFROLEMAP	"rad_vfrolemap"
#define	PAM_RADIUS_HOMEAD	"rad_homead"
#define	PAM_RADIUS_HOMEVF	"rad_homevf"
#define	PAM_RADIUS_FIPSMODE	"rad_fipsmode"
#define	PAM_RADIUS_CERT_VERIFY	"rad_verification_return"
#define	PAM_RADIUS_IMPROPER_SYNTAX		100
#define	PAM_RADIUS_INVALID_ROLE			101
#define	PAM_RADIUS_PASSWORD_EXPIRED		102
#define	PAM_RADIUS_COMM_FAILED			103

#define	INVALID_VF			-1
#define	INVALID_CHASSIS		-1
#define PEAP_PROTOCOL		2

#define PAM_AUTH_SECUREID	-2

/* It needs to be updated whenever FOS updates MAX no of vfs. */ 
#define	PAM_RADIUS_MAXVF	256
/* These 2 needs to be updated whenever FOS updates default context .*/ 
#define CHASSIS_ID			255	

/* Months and Max days. */
typedef	enum {
	JAN	= 1,
	FEB,
	MAR,
	APR,
	MAY,
	JUN,
	JUL,
	AUG,
	SEP,
	OCT,
	NOV,
	DEC,
	TWENTYEIGHT = 28,
	THIRTY = 30,
	THIRTYONE = 31
} months_maxdays;

extern int
pam_peap_authenticate(const char *, const char *, radius_conf_t*, AUTH_HDR *);

int fipsmode = 0;

/* Module defines */
#ifndef BUFFER_SIZE
#define BUFFER_SIZE      1024
#endif /* BUFFER_SIZE */
#define MAXPWNAM 40     /* maximum user name length. Server dependent,
                         * this is the default value
                         */
#define MAXPASS 128     /* max password length. Again, depends on server
                         * compiled in. This is the default.
                         */
#ifndef CONF_FILE       /* the configuration file holding the server secret */
#define CONF_FILE       "/etc/raddb/server"
#endif /* CONF_FILE */

#ifndef FALSE
#define FALSE 0
#undef TRUE
#define TRUE !FALSE
#endif

#endif /* PAM_RADIUS_H */
